/*
	Author   : Tim Ruehsen, Crisi
   Project  : IRA  -  680x0 Interactive ReAssembler
	Part     : IRA.h
   Purpose  : Contains definitions for all IRA sources.
   Version  : $VER: IRA.h V1.05
   Date     : 16.05.1994
	Copyright: (C)1993-1995 Tim Ruehsen
*/

#ifndef IRA_H
#define IRA_H


#define VERSION     "2"
#define REVISION    "00"
//#define BETA        1
#if BETA == 0
  #define IDSTRING1   ("\nIRA-pc (680x0 Reassembler) v%s.%s "__DATE__" (c)1999 Antiriad\n\n")
  #define IDSTRING2   ("; IRA-pc (680x0 Reassembler) v%s.%s "__DATE__" (c)1999 Antiriad\n; original by Tim Ruehsen (SiliconSurfer/PHANTASM)\n\n")
#else
  #define IDSTRING1   ("\nIRA-pc (680x0 Reassembler) v%s.%sbeta "__DATE__" (c)1999 Antiriad\n\n")
  #define IDSTRING2   ("; IRA-pc (680x0 Reassembler) v%s.%sbeta "__DATE__" (c)1999 Antiriad\n; original by Tim Ruehsen (SiliconSurfer/PHANTASM)\n\n")
#endif

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
//#include <functions.h>
#include <ctype.h>
#include <string.h>
//#include <intuition/intuitionbase.h>
//#include <exec/memory.h>
//#include <exec/nodes.h>
//#include <libraries/dosextens.h>

#include "deftypes.h"

/*  Anzahl Eintraege in x_adrs  309  */
/* #define ADRCOUNT          (sizeof(x_adrs)/sizeof(struct x_adr)) */

#define ADR_OUTPUT        (1<<0)  /* Adressen im Codebereich ausgeben */
#define KEEP_BINARY       (1<<1)  /* Binaeres Zwischenfile loeschen   */
#define SHOW_RELOCINFO    (1<<2)  /* Relokations-Informationen zeigen */
#define KEEP_ZEROHUNKS    (1<<3)  /* Null-Hunks mit ausgeben          */
#define OLDSTYLE          (1<<4)  /* Use oldstyle EA-formats          */
#define SPLITFILE         (1<<5)  /* Put sections into own files      */
#define BASEREG1          (1<<6)  /* Address proposals for d16(Ax)    */
#define BASEREG2          (1<<7)  /* Base-relative mode d16(Ax)       */
#define PREPROC           (1<<8)  /* To find data in code sections    */
#define CONFIG            (1<<9)  /* Configfile should be included    */
#define FORCECODE         (1<<10) /* Mark areas ending with RTS as code */
#define ROMTAGatZERO      (1<<11) /* Don't assume a code entry at adr=0 */

#define M68000            1
#define M68010            2
#define M68020            4
#define M68030            8
#define M68040            16
#define M68060            32
#define M68851           128
#define M68881           256
#define M680x0            (1+2+4+8+16+32)
#define M010UP            (2+4+8+16+32)
#define M020UP            (4+8+16+32)
#define M030UP            (8+16+32)
#define M040UP            (16+32)


#define OPC_BITFIELD       0
#define OPC_BITSHIFT1      1
#define OPC_BITSHIFT2      2
#define OPC_RTE            6
#define OPC_RTR            7
#define OPC_RTS            8
#define OPC_RTD            9
#define OPC_DIVL          24
#define OPC_MULL          25
#define OPC_JMP           27
#define OPC_JSR           28
#define OPC_PEA           29
#define OPC_MOVEM1        35
#define OPC_MOVEM3        37
#define OPC_LEA           39
#define OPC_TST           43
#define OPC_PACK1         67
#define OPC_PACK2         68
#define OPC_UNPK1         69
#define OPC_UNPK2         70
#define OPC_MOVEB         77
#define OPC_MOVEW         79
#define OPC_MOVEAL        80
#define OPC_MOVEL         81
#define OPC_RTM           96
#define OPC_CALLM         97
#define OPC_C2            98
#define OPC_CMPI          99
#define OPC_MOVES        106
#define OPC_DBCC         107
#define OPC_BCC          114
#define OPC_MOVE162      116


#define NOADRMODE         99     /* Addressmode for DC.W */

struct x_adr {
	char name[12];
	ULONG adr;
	};

#define STDNAMELENGTH   256

#define MEMF_CLEAR	(1)

void *AllocMem(ULONG size, ULONG flags);

void FreeMem(void *ptr,ULONG cnt);

// int to ascii (signed, decimal)

char *ira_itoa(ULONG ta);

// int to ascii (signed, decimal)

char *itohex(ULONG ta, ULONG digits);

// copy size longwords from src to dst (with overlapping support)

void lmovmem(ULONG *src, ULONG *dst, ULONG size);

char mnebuf[256];
char adrbuf[256];
char dtabuf[256];

#define mnecat(a)	strcat(mnebuf, a)
#define adrcat(a)	strcat(adrbuf, a)
#define dtacat(a)	strcat(dtabuf, a)

void mnecasecat(char *src);
void adrcasecat(char *src);
void dtacasecat(char *src);

// Intel Read macros
// -----------------

#define ReadByte(a)	(*(UBYTE *)(a))
#define ReadWord(a)	(*(UWORD *)(a))
#define ReadLong(a)	(*(ULONG *)(a))

// Intel Write macros
// ------------------

#define WriteByte(a,d)	(*(UBYTE *)(a) = (d))
#define WriteWord(a,d)	(*(UWORD *)(a) = (d))
#define WriteLong(a,d)	(*(ULONG *)(a) = (d))

// 680x0 Read macros
// -----------------

static inline ULONG GetWord(UWORD *a)
{
   ULONG retval;
   asm("xorl %k0,%k0\n\tmovw %w1,%w0\n\trolw $8,%w0" : "=&r" (retval) : "m" (*a) : "cc");
   return retval;
}
static inline ULONG GetLong(ULONG *a)
{
   ULONG retval;
   asm("bswapl %0" : "=r" (retval) : "0" (*a) : "cc");
   return retval;
}

#define ReadByte68k(a) ReadByte(a)
#define ReadWord68k(a) GetWord((UWORD *)a)
#define ReadLong68k(a) GetLong((ULONG *)a)

// 680x0 Write macros
// ------------------

static inline void PutWord(UWORD *a, ULONG v)
{
   asm("rolw $8,%0" : "=r" (v) : "0" (v) : "cc");
   *a = v;
}
static inline void PutLong(ULONG *a, ULONG v)
{
   asm("bswapl %0" : "=r" (v) : "0" (v) : "cc");
   *a = v;
}

#define WriteByte68k(a,d) WriteByte(a,d)
#define WriteWord68k(a,d) PutWord((UWORD *)a,d)
#define WriteLong68k(a,d) PutLong((ULONG *)a,d)

int upper_case;

#endif



